# This file is autogenerated by medikit code generator.
# All changes will be overwritten.

from setuptools import setup, find_packages
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Py3 compatibility hacks, borrowed from IPython.
try:
    execfile
except NameError:

    def execfile(fname, globs, locs=None):
        locs = locs or globs
        exec(compile(open(fname).read(), fname, "exec"), globs, locs)


# Get the long description from the README file
try:
    with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
        long_description = f.read()
except:
    long_description = ''

# Get the classifiers from the classifiers file
tolines = lambda c: list(filter(None, map(lambda s: s.strip(), c.split('\n'))))
try:
    with open(path.join(here, 'classifiers.txt'), encoding='utf-8') as f:
        classifiers = tolines(f.read())
except:
    classifiers = []

version_ns = {}
try:
    execfile(path.join(here, 'whistle/_version.py'), version_ns)
except EnvironmentError:
    version = 'dev'
else:
    version = version_ns.get('__version__', 'dev')

setup(
    author='Romain Dorgueil',
    author_email='romain@dorgueil.net',
    description='Event dispatcher for Python 3.5+',
    license='Apache License, Version 2.0',
    name='whistle',
    version=version,
    long_description=long_description,
    classifiers=classifiers,
    packages=find_packages(exclude=['ez_setup', 'example', 'test']),
    include_package_data=True,
    extras_require={
        'dev':
        ['coverage (>= 4.4, < 5.0)', 'pytest (>= 3.1, < 4.0)', 'pytest-cov (>= 2.5, < 3.0)', 'sphinx (>= 1.6, < 2.0)']
    },
    url='https://github.com/python-whistle/whistle',
    download_url='https://github.com/python-whistle/whistle/tarball/{version}'.format(version=version),
)



import operator

from whistle.event import Event


class EventDispatcher(object):
    """
    A logical event dispatcher. All events can only be dispatched in the context of an :class:`EventDispatcher`, and
    event dispatchers are fully independant and isolated.

    """

    def __init__(self):
        self._listeners = {}
        self._sorted = {}

    def dispatch(self, event_id, event=None):
        if event is None:
            event = Event()

        event.dispatcher = self
        event.name = event_id

        if not event_id in self._listeners:
            return event

        self.do_dispatch(self.get_listeners(event_id), event)

        return event

    def get_listeners(self, event_id=None):
        if event_id is not None:
            if not event_id in self._sorted:
                self.sort_listeners(event_id)

            return self._sorted[event_id]

        for event_id in self._listeners:
            if not event_id in self._sorted:
                self.sort_listeners(event_id)

        return self._sorted

    def has_listeners(self, event_id=None):
        return bool(len(self.get_listeners(event_id)))

    def add_listener(self, event_id, listener, priority=0):
        if not event_id in self._listeners:
            self._listeners[event_id] = {}
        if not priority in self._listeners[event_id]:
            self._listeners[event_id][priority] = []
        self._listeners[event_id][priority].append(listener)

        if event_id in self._sorted:
            del self._sorted[event_id]

    def listen(self, event_id, priority=0):
        """
        Decorator that add the decorated functions as one of this instance listeners, for the given event name.

        :param event_id:
        :param priority:
        :return:

        """

        def wrapper(listener):
            self.add_listener(event_id, listener, priority)
            return listener

        return wrapper

    def remove_listener(self, event_id, listener):
        """
        Remove a fiven listener from this event dispatcher. For now, if the listener is not registered for this event,
        this method has no effect, but we may raise ValueError in the future if the given listener is not found, you
        should catch it if you're not certain the listener is registered.

        TODO raise ValueError if not found.

        :param event_id:
        :param listener:
        :return:
        """
        if not event_id in self._listeners:
            return

        for priority, listeners in self._listeners[event_id].items():
            if listener in self._listeners[event_id][priority]:
                # pylint: disable=filter-builtin-not-iterating
                self._listeners[event_id][priority] = filter(
                    lambda l: l != listener, self._listeners[event_id][priority]
                )
                if event_id in self._sorted:
                    del self._sorted[event_id]

    def do_dispatch(self, listeners, event):
        for listener in listeners:
            listener(event)
            if event.propagation_stopped: break

    def sort_listeners(self, event_id):
        self._sorted[event_id] = []
        if event_id in self._listeners:
            self._sorted[event_id] = [
                listener
                for listeners in sorted(self._listeners[event_id].items(), key=operator.itemgetter(0))
                for listener in listeners[1]
            ]
